
"""
Contributors can be viewed at:
http://svn.secondlife.com/svn/linden/projects/2008/pyogp/lib/base/trunk/CONTRIBUTORS.txt 

$LicenseInfo:firstyear=2008&license=apachev2$

Copyright 2009, Linden Research, Inc.

Licensed under the Apache License, Version 2.0.
You may obtain a copy of the License at:
    http://www.apache.org/licenses/LICENSE-2.0
or in 
    http://svn.secondlife.com/svn/linden/projects/2008/pyogp/lib/base/LICENSE.txt

$/LicenseInfo$
"""

class Param(object):
    """
    a visual parameter of an agent
    """
    def __init__(self, id, group, name, wearable, value_default, value_min,
                 value_max, value, label, label_min, label_max):

        self.id = id
        self.group = group
        self.name = name
        self.wearable = wearable
        self.value_default = float(value_default)
        self.value_min = float(value_min)
        self.value_max = float(value_max)
        self.value = float(value)
        self.label = label
        self.label_min = label_min
        self.label_max = label_max

    def floatToByte(self):
        """
        Converts value from a float to a byte
        """
        result = self.value - self.value_min
        result /= self.value_max - self.value_min
        return int(255*result)

    def byteToFloat(self, val):
        """
        Converts val from a byte to a float and stores its to value
        """        
        result = val * (1.0/255)
        result *= self.value_max - self.value_min
        result += self.value_min
        self.value = round(result, 3)

class VisualParams(object):
    """
    The visual params of an agent
    """
    params = {}
    def __init__(self):
        self.params[1] = Param(1, 0, u'Big_Brow', u'shape', 0.0, -.3, 2, 0.0, u'Brow Size', u'Small', u'Large')
        self.params[2] = Param(2, 0, u'Nose_Big_Out', u'shape', 0.0, -0.8, 2.5, 0.0, u'Nose Size', u'Small', u'Large')
        self.params[4] = Param(4, 0, u'Broad_Nostrils', u'shape', 0.0, -.5, 1, 0.0, u'Nostril Width', u'Narrow', u'Broad')
        self.params[5] = Param(5, 0, u'Cleft_Chin', u'shape', 0.0, -.1, 1, 0.0, u'Chin Cleft', u'Round', u'Cleft')
        self.params[6] = Param(6, 0, u'Bulbous_Nose_Tip', u'shape', 0.0, -.3, 1, 0.0, u'Nose Tip Shape', u'Pointy', u'Bulbous')
        self.params[7] = Param(7, 0, u'Weak_Chin', u'shape', 0.0, -.5, .5, 0.0, u'Chin Angle', u'Chin Out', u'Chin In')
        self.params[8] = Param(8, 0, u'Double_Chin', u'shape', 0.0, -.5, 1.5, 0.0, u'Chin-Neck', u'Tight Chin', u'Double Chin')
        self.params[10] = Param(10, 0, u'Sunken_Cheeks', u'shape', 0.0, -1.5, 3, 0.0, u'Lower Cheeks', u'Well-Fed', u'Sunken')
        self.params[11] = Param(11, 0, u'Noble_Nose_Bridge', u'shape', 0.0, -.5, 1.5, 0.0, u'Upper Bridge', u'Low', u'High')
        self.params[12] = Param(12, 0, u'Jowls', u'shape', 0.0, -.5, 2.5, 0.0, '', u'Less', u'More')
        self.params[13] = Param(13, 0, u'Cleft_Chin_Upper', u'shape', 0.0, 0, 1.5, 0.0, u'Upper Chin Cleft', u'Round', u'Cleft')
        self.params[14] = Param(14, 0, u'High_Cheek_Bones', u'shape', 0.0, -.5, 1, 0.0, u'Cheek Bones', u'Low', u'High')
        self.params[15] = Param(15, 0, u'Ears_Out', u'shape', 0.0, -.5, 1.5, 0.0, u'Ear Angle', u'In', u'Out')
        self.params[16] = Param(16, 0, u'Pointy_Eyebrows', u'hair', 0.0, -.5, 3, 0.0, u'Eyebrow Points', u'Smooth', u'Pointy')
        self.params[17] = Param(17, 0, u'Square_Jaw', u'shape', 0.0, -.5, 1, 0.0, u'Jaw Shape', u'Pointy', u'Square')
        self.params[18] = Param(18, 0, u'Puffy_Upper_Cheeks', u'shape', 0.0, -1.5, 2.5, 0.0, u'Upper Cheeks', u'Thin', u'Puffy')
        self.params[19] = Param(19, 0, u'Upturned_Nose_Tip', u'shape', 0.0, -1.5, 1, 0.0, u'Nose Tip Angle', u'Downturned', u'Upturned')
        self.params[20] = Param(20, 0, u'Bulbous_Nose', u'shape', 0.0, -.5, 1.5, 0.0, u'Nose Thickness', u'Thin Nose', u'Bulbous Nose')
        self.params[21] = Param(21, 0, u'Upper_Eyelid_Fold', u'shape', 0.0, -0.2, 1.3, 0.0, u'Upper Eyelid Fold', u'Uncreased', u'Creased')
        self.params[22] = Param(22, 0, u'Attached_Earlobes', u'shape', 0.0, 0, 1, 0.0, u'Attached Earlobes', u'Unattached', u'Attached')
        self.params[23] = Param(23, 0, u'Baggy_Eyes', u'shape', 0.0, -.5, 1.5, 0.0, u'Eye Bags', u'Smooth', u'Baggy')
        self.params[24] = Param(24, 0, u'Wide_Eyes', u'shape', 0.0, -1.5, 2, 0.0, u'Eye Opening', u'Narrow', u'Wide')
        self.params[25] = Param(25, 0, u'Wide_Lip_Cleft', u'shape', 0.0, -.8, 1.5, 0.0, u'Lip Cleft', u'Narrow', u'Wide')
        self.params[26] = Param(26, 1, u'Lips_Thin', u'shape', 0.0, 0, .7, 0.0, '', '', '')
        self.params[27] = Param(27, 0, u'Wide_Nose_Bridge', u'shape', 0.0, -1.3, 1.2, 0.0, u'Bridge Width', u'Narrow', u'Wide')
        self.params[28] = Param(28, 1, u'Lips_Fat', u'shape', 0.0, 0, 2, 0.0, '', '', '')
        self.params[29] = Param(29, 1, u'Wide_Upper_Lip', u'shape', 0.0, -.7, 1.3, 0.0, '', '', '')
        self.params[30] = Param(30, 1, u'Wide_Lower_Lip', u'shape', 0.0, -.7, 1.3, 0.0, '', '', '')
        self.params[31] = Param(31, 0, u'Arced_Eyebrows', u'hair', .5, 0, 2, .5, u'Eyebrow Arc', u'Flat', u'Arced')
        self.params[32] = Param(32, 1, u'Male_Skeleton', u'shape', 0.0, 0, 1, 0.0, '', u'Female', u'Male')
        self.params[33] = Param(33, 0, u'Height', u'shape', 0.0, -2.3, 2, 0.0, u'Height', u'Short', u'Tall')
        self.params[34] = Param(34, 0, u'Thickness', u'shape', 0.0, -0.7, 1.5, 0.0, u'Body Thickness', u'Body Thin', u'Body Thick')
        self.params[35] = Param(35, 0, u'Big_Ears', u'shape', 0.0, -1, 2, 0.0, u'Ear Size', u'Small', u'Large')
        self.params[36] = Param(36, 0, u'Shoulders', u'shape', -0.5, -1.8, 1.4, -0.5, u'Shoulders', u'Narrow', u'Broad')
        self.params[37] = Param(37, 0, u'Hip Width', u'shape', 0.0, -3.2, 2.8, 0.0, u'Hip Width', u'Narrow', u'Wide')
        self.params[38] = Param(38, 0, u'Torso Length', u'shape', 0.0, -1, 1, 0.0, '', u'Short Torso', u'Long Torso')
        self.params[40] = Param(40, 1, u'Male_Head', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[41] = Param(41, 1, u'Old', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[51] = Param(51, 1, u'Furrowed_Eyebrows', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[53] = Param(53, 1, u'Surprised_Eyebrows', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[54] = Param(54, 1, u'Worried_Eyebrows', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[55] = Param(55, 1, u'Frown_Mouth', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[57] = Param(57, 1, u'Smile_Mouth', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[58] = Param(58, 1, u'Blink_Left', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[59] = Param(59, 1, u'Blink_Right', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[70] = Param(70, 1, u'Lipsync_Aah', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[71] = Param(71, 1, u'Lipsync_Ooh', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[80] = Param(80, 0, u'male', u'shape', 0.0, 0, 1, 0.0, '', '', '')
        self.params[93] = Param(93, 0, u'Glove Length', u'gloves', .8, .01, 1, .8, '', u'Short', u'Long')
        self.params[98] = Param(98, 0, u'Eye Lightness', u'eyes', 0.0, 0, 1, 0.0, '', u'Darker', u'Lighter')
        self.params[99] = Param(99, 0, u'Eye Color', u'eyes', 0, 0, 1, 0, '', u'Natural', u'Unnatural')
        self.params[100] = Param(100, 1, u'Male_Torso', '', 0.0, 0, 1, 0.0, '', u'Male_Torso', '')
        self.params[101] = Param(101, 1, u'Hands_Relaxed', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[102] = Param(102, 1, u'Hands_Point', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[103] = Param(103, 1, u'Hands_Fist', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[104] = Param(104, 1, u'Big_Belly_Torso', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[105] = Param(105, 0, u'Breast Size', u'shape', .5, 0, 1, .5, '', u'Small', u'Large')
        self.params[106] = Param(106, 1, u'Muscular_Torso', u'shape', 0.0, 0, 1.4, 0.0, u'Torso Muscles', u'Regular', u'Muscular')
        self.params[108] = Param(108, 0, u'Rainbow Color', u'skin', 0.0, 0, 1, 0.0, '', u'None', u'Wild')
        self.params[110] = Param(110, 0, u'Red Skin', u'skin', 0.0, 0, 0.1, 0.0, u'Ruddiness', u'Pale', u'Ruddy')
        self.params[111] = Param(111, 0, u'Pigment', u'skin', .5, 0, 1, .5, '', u'Light', u'Dark')
        self.params[112] = Param(112, 0, u'Rainbow Color', u'hair', 0.0, 0, 1, 0.0, '', u'None', u'Wild')
        self.params[113] = Param(113, 0, u'Red Hair', u'hair', 0.0, 0, 1, 0.0, '', u'No Red', u'Very Red')
        self.params[114] = Param(114, 0, u'Blonde Hair', u'hair', .5, 0, 1, .5, '', u'Black', u'Blonde')
        self.params[115] = Param(115, 0, u'White Hair', u'hair', 0.0, 0, 1, 0.0, '', u'No White', u'All White')
        self.params[116] = Param(116, 0, u'Rosy Complexion', u'skin', 0.0, 0, 1, 0.0, '', u'Less Rosy', u'More Rosy')
        self.params[117] = Param(117, 0, u'Lip Pinkness', u'skin', 0.0, 0, 1, 0.0, '', u'Darker', u'Pinker')
        self.params[118] = Param(118, 1, u'Wrinkles', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[119] = Param(119, 0, u'Eyebrow Size', u'hair', 0.5, 0, 1, 0.5, '', u'Thin Eyebrows', u'Bushy Eyebrows')
        self.params[125] = Param(125, 1, u'Shading', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[126] = Param(126, 1, u'Shading', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[130] = Param(130, 0, u'Front Fringe', u'hair', .45, 0, 1, .45, '', u'Short', u'Long')
        self.params[131] = Param(131, 0, u'Side Fringe', u'hair', .5, 0, 1, .5, '', u'Short', u'Long')
        self.params[132] = Param(132, 0, u'Back Fringe', u'hair', .39, 0, 1, .39, '', u'Short', u'Long')
        self.params[133] = Param(133, 0, u'Hair Front', u'hair', .25, 0, 1, .25, '', u'Short', u'Long')
        self.params[134] = Param(134, 0, u'Hair Sides', u'hair', .5, 0, 1, .5, '', u'Short', u'Long')
        self.params[135] = Param(135, 0, u'Hair Back', u'hair', .55, 0, 1, .55, '', u'Short', u'Long')
        self.params[136] = Param(136, 0, u'Hair Sweep', u'hair', .5, 0, 1, .5, '', u'Sweep Forward', u'Sweep Back')
        self.params[137] = Param(137, 0, u'Hair Tilt', u'hair', .5, 0, 1, .5, '', u'Left', u'Right')
        self.params[140] = Param(140, 0, u'Hair_Part_Middle', u'hair', 0.0, 0, 2, 0.0, u'Middle Part', u'No Part', u'Part')
        self.params[141] = Param(141, 0, u'Hair_Part_Right', u'hair', 0.0, 0, 2, 0.0, u'Right Part', u'No Part', u'Part')
        self.params[142] = Param(142, 0, u'Hair_Part_Left', u'hair', 0.0, 0, 2, 0.0, u'Left Part', u'No Part', u'Part')
        self.params[143] = Param(143, 0, u'Hair_Sides_Full', u'hair', 0.125, -4, 1.5, 0.125, u'Full Hair Sides', u'Mowhawk', u'Full Sides')
        self.params[144] = Param(144, 1, u'Bangs_Front_Up', u'hair', 0.0, 0, 1, 0.0, u'Front Bangs Up', u'Bangs', u'Bangs Up')
        self.params[145] = Param(145, 1, u'Bangs_Front_Down', u'hair', 0.0, 0, 5, 0.0, u'Front Bangs Down', u'Bangs', u'Bangs Down')
        self.params[146] = Param(146, 1, u'Bangs_Sides_Up', u'hair', 0.0, 0, 1, 0.0, u'Side Bangs Up', u'Side Bangs', u'Side Bangs Up')
        self.params[147] = Param(147, 1, u'Bangs_Sides_Down', u'hair', 0.0, 0, 2, 0.0, u'Side Bangs Down', u'Side Bangs', u'Side Bangs Down')
        self.params[148] = Param(148, 1, u'Bangs_Back_Up', u'hair', 0.0, 0, 1, 0.0, u'Back Bangs Up', u'Back Bangs', u'Back Bangs Up')
        self.params[149] = Param(149, 1, u'Bangs_Back_Down', u'hair', 0.0, 0, 2, 0.0, u'Back Bangs Down', u'Back Bangs', u'Back Bangs Down')
        self.params[150] = Param(150, 0, u'Body Definition', u'skin', 0, 0, 1, 0, '', u'Less', u'More')
        self.params[151] = Param(151, 1, u'Big_Butt_Legs', u'shape', 0.0, 0, 1, 0.0, u'Butt Size', u'Regular', u'Large')
        self.params[152] = Param(152, 1, u'Muscular_Legs', u'shape', 0.0, 0, 1.5, 0.0, u'Leg Muscles', u'Regular Muscles', u'More Muscles')
        self.params[153] = Param(153, 1, u'Male_Legs', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[155] = Param(155, 0, u'Lip Width', u'shape', 0, -0.9, 1.3, 0, u'Lip Width', u'Narrow Lips', u'Wide Lips')
        self.params[156] = Param(156, 1, u'Big_Belly_Legs', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[157] = Param(157, 0, u'Belly Size', u'shape', 0, 0, 1, 0, '', u'Small', u'Big')
        self.params[158] = Param(158, 1, u'Shading', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[159] = Param(159, 1, u'Shading', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[160] = Param(160, 1, u'Shading', u'pants', 0.0, 0, 1, 0.0, '', '', '')
        self.params[161] = Param(161, 1, u'Shading', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[162] = Param(162, 0, u'Facial Definition', u'skin', 0, 0, 1, 0, '', u'Less', u'More')
        self.params[163] = Param(163, 0, u'wrinkles', u'skin', 0, 0, 1, 0, '', u'Less', u'More')
        self.params[165] = Param(165, 0, u'Freckles', u'skin', 0.0, 0, 1, 0.0, '', u'Less', u'More')
        self.params[166] = Param(166, 0, u'Sideburns', u'hair', 0.0, 0, 1, 0.0, '', u'Short Sideburns', u'Mutton Chops')
        self.params[167] = Param(167, 0, u'Moustache', u'hair', 0.0, 0, 1, 0.0, '', u'Chaplin', u'Handlebars')
        self.params[168] = Param(168, 0, u'Soulpatch', u'hair', 0.0, 0, 1, 0.0, '', u'Less soul', u'More soul')
        self.params[169] = Param(169, 0, u'Chin Curtains', u'hair', 0.0, 0, 1, 0.0, '', u'Less Curtains', u'More Curtains')
        self.params[171] = Param(171, 1, u'Hair_Front_Down', u'hair', 0.0, 0, 1, 0.0, u'Front Hair Down', u'Front Hair', u'Front Hair Down')
        self.params[172] = Param(172, 1, u'Hair_Front_Up', u'hair', 0.0, 0, 1, 0.0, u'Front Hair Up', u'Front Hair', u'Front Hair Up')
        self.params[173] = Param(173, 1, u'Hair_Sides_Down', u'hair', 0.0, 0, 1, 0.0, u'Sides Hair Down', u'Sides Hair', u'Sides Hair Down')
        self.params[174] = Param(174, 1, u'Hair_Sides_Up', u'hair', 0.0, 0, 1, 0.0, u'Sides Hair Up', u'Sides Hair', u'Sides Hair Up')
        self.params[175] = Param(175, 1, u'Hair_Back_Down', u'hair', 0.0, 0, 3, 0.0, u'Back Hair Down', u'Back Hair', u'Back Hair Down')
        self.params[176] = Param(176, 1, u'Hair_Back_Up', u'hair', 0.0, 0, 1, 0.0, u'Back Hair Up', u'Back Hair', u'Back Hair Up')
        self.params[177] = Param(177, 0, u'Hair_Rumpled', u'hair', 0.0, 0, 1, 0.0, u'Rumpled Hair', u'Smooth Hair', u'Rumpled Hair')
        self.params[178] = Param(178, 1, u'Hair_Swept_Back', u'hair', 0.0, 0, 1, 0.0, u'Swept Back Hair', u'NotHair', u'Swept Back')
        self.params[179] = Param(179, 1, u'Hair_Swept_Forward', u'hair', 0.0, 0, 1, 0.0, u'Swept Forward Hair', u'Hair', u'Swept Forward')
        self.params[180] = Param(180, 1, u'Hair_Volume', u'hair', 0.0, 0, 1.3, 0.0, u'Hair Volume', u'Less', u'More')
        self.params[181] = Param(181, 0, u'Hair_Big_Front', u'hair', 0.14, -1, 1, 0.14, u'Big Hair Front', u'Less', u'More')
        self.params[182] = Param(182, 0, u'Hair_Big_Top', u'hair', .7, -1, 1, .7, u'Big Hair Top', u'Less', u'More')
        self.params[183] = Param(183, 0, u'Hair_Big_Back', u'hair', 0.05, -1, 1, 0.05, u'Big Hair Back', u'Less', u'More')
        self.params[184] = Param(184, 0, u'Hair_Spiked', u'hair', 0.0, 0, 1, 0.0, u'Spiked Hair', u'No Spikes', u'Big Spikes')
        self.params[185] = Param(185, 0, u'Deep_Chin', u'shape', 0.0, -1, 1, 0.0, u'Chin Depth', u'Shallow', u'Deep')
        self.params[186] = Param(186, 1, u'Egg_Head', u'shape', 0.0, -1.3, 1, 0.0, u'Egg Head', u'Chin Heavy', u'Forehead Heavy')
        self.params[187] = Param(187, 1, u'Squash_Stretch_Head', u'shape', 0.0, -.5, 1, 0.0, u'Squash/Stretch Head', u'Squash Head', u'Stretch Head')
        self.params[188] = Param(188, 1, u'Square_Head', u'shape', 0.0, 0, .7, 0.0, '', u'Less Square', u'More Square')
        self.params[189] = Param(189, 1, u'Round_Head', u'shape', 0.0, 0, 1, 0.0, '', u'Less Round', u'More Round')
        self.params[190] = Param(190, 1, u'Hair_Tilt_Right', u'hair', 0.0, 0, 1, 0.0, u'Hair Tilted Right', u'Hair', u'Tilt Right')
        self.params[191] = Param(191, 1, u'Hair_Tilt_Left', u'hair', 0.0, 0, 1, 0.0, u'Hair Tilted Left', u'Hair', u'Tilt Left')
        self.params[192] = Param(192, 0, u'Bangs_Part_Middle', u'hair', 0.0, 0, 1, 0.0, u'Part Bangs', u'No Part', u'Part Bangs')
        self.params[193] = Param(193, 0, u'Head Shape', u'shape', .5, 0, 1, .5, u'Head Shape', u'More Square', u'More Round')
        self.params[194] = Param(194, 1, u'Eye_Spread', '', 0.0, -2, 2, 0.0, '', u'Eyes Together', u'Eyes Spread')
        self.params[195] = Param(195, 1, u'EyeBone_Spread', u'shape', 0.0, -1, 1, 0.0, '', u'Eyes Together', u'Eyes Spread')
        self.params[196] = Param(196, 0, u'Eye Spacing', u'shape', 0, -2, 1, 0, u'Eye Spacing', u'Close Set Eyes', u'Far Set Eyes')
        self.params[197] = Param(197, 1, u'Shoe_Heels', u'shoes', 0.0, 0, 1, 0.0, '', u'No Heels', u'High Heels')
        self.params[198] = Param(198, 0, u'Heel Height', u'shoes', 0, 0, 1, 0, '', u'Low Heels', u'High Heels')
        self.params[300] = Param(300, 1, u'Express_Closed_Mouth', '', 1, 0, 1, 1, '', '', '')
        self.params[301] = Param(301, 1, u'Express_Tongue_Out', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[302] = Param(302, 1, u'Express_Surprise_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[303] = Param(303, 1, u'Express_Wink_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[304] = Param(304, 1, u'Express_Embarrassed_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[305] = Param(305, 1, u'Express_Shrug_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[306] = Param(306, 1, u'Express_Kiss', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[307] = Param(307, 1, u'Express_Bored_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[308] = Param(308, 1, u'Express_Repulsed_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[309] = Param(309, 1, u'Express_Disdain', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[310] = Param(310, 1, u'Express_Afraid_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[311] = Param(311, 1, u'Express_Worry_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[312] = Param(312, 1, u'Express_Cry_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[313] = Param(313, 1, u'Express_Sad_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[314] = Param(314, 1, u'Express_Anger_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[315] = Param(315, 1, u'Express_Frown', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[316] = Param(316, 1, u'Express_Laugh_Emote', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[317] = Param(317, 1, u'Express_Toothsmile', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[318] = Param(318, 1, u'Express_Smile', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[400] = Param(400, 1, u'Displace_Hair_Facial', u'hair', 0.0, 0, 2, 0.0, u'Hair Thickess', u'Cropped Hair', u'Bushy Hair')
        self.params[500] = Param(500, 1, u'Shoe_Heel_Height', u'shoes', 0.0, 0, 1, 0.0, u'Heel Height', u'Low Heels', u'High Heels')
        self.params[501] = Param(501, 1, u'Shoe_Platform_Height', u'shoes', 0.0, 0, 1, 0.0, u'Platform Height', u'Low Platforms', u'High Platforms')
        self.params[502] = Param(502, 1, u'Shoe_Platform', u'shoes', 0.0, 0, 1, 0.0, '', u'No Heels', u'High Heels')
        self.params[503] = Param(503, 0, u'Platform Height', u'shoes', 0, 0, 1, 0, '', u'Low Platforms', u'High Platforms')
        self.params[505] = Param(505, 0, u'Lip Thickness', u'shape', .5, 0, 1, .5, '', u'Thin Lips', u'Fat Lips')
        self.params[506] = Param(506, 0, u'Mouth_Height', u'shape', 0.0, -2, 2, 0.0, u'Mouth Position', u'High', u'Low')
        self.params[507] = Param(507, 0, u'Breast_Gravity', u'shape', 0, -1.5, 2, 0, u'Breast Buoyancy', u'Less Gravity', u'More Gravity')
        self.params[508] = Param(508, 0, u'Shoe_Platform_Width', u'shoes', 0.0, -1, 2, 0.0, u'Platform Width', u'Narrow', u'Wide')
        self.params[509] = Param(509, 1, u'Shoe_Heel_Point', u'shoes', 0.0, 0, 1, 0.0, u'Heel Shape', u'Default Heels', u'Pointy Heels')
        self.params[510] = Param(510, 1, u'Shoe_Heel_Thick', u'shoes', 0.0, 0, 1, 0.0, u'Heel Shape', u'default Heels', u'Thick Heels')
        self.params[511] = Param(511, 1, u'Shoe_Toe_Point', u'shoes', 0.0, 0, 1, 0.0, u'Toe Shape', u'Default Toe', u'Pointy Toe')
        self.params[512] = Param(512, 1, u'Shoe_Toe_Square', u'shoes', 0.0, 0, 1, 0.0, u'Toe Shape', u'Default Toe', u'Square Toe')
        self.params[513] = Param(513, 0, u'Heel Shape', u'shoes', .5, 0, 1, .5, '', u'Pointy Heels', u'Thick Heels')
        self.params[514] = Param(514, 0, u'Toe Shape', u'shoes', .5, 0, 1, .5, '', u'Pointy', u'Square')
        self.params[515] = Param(515, 0, u'Foot_Size', u'shape', 0.0, -1, 3, 0.0, u'Foot Size', u'Small', u'Big')
        self.params[516] = Param(516, 1, u'Displace_Loose_Lowerbody', u'pants', 0, 0, 1, 0, u'Pants Fit', '', '')
        self.params[517] = Param(517, 0, u'Wide_Nose', u'shape', 0.0, -.5, 1, 0.0, u'Nose Width', u'Narrow', u'Wide')
        self.params[518] = Param(518, 0, u'Eyelashes_Long', u'shape', 0.0, -.3, 1.5, 0.0, u'Eyelash Length', u'Short', u'Long')
        self.params[600] = Param(600, 1, u'Sleeve Length Cloth', u'shirt', .7, 0, 0.85, .7, '', '', '')
        self.params[601] = Param(601, 1, u'Shirt Bottom Cloth', u'shirt', .8, 0, 1, .8, '', '', '')
        self.params[602] = Param(602, 1, u'Collar Front Height Cloth', u'shirt', .8, 0, 1, .8, '', '', '')
        self.params[603] = Param(603, 0, u'Sleeve Length', u'undershirt', .4, .01, 1, .4, '', u'Short', u'Long')
        self.params[604] = Param(604, 0, u'Bottom', u'undershirt', .85, 0, 1, .85, '', u'Short', u'Long')
        self.params[605] = Param(605, 0, u'Collar Front', u'undershirt', .84, 0, 1, .84, '', u'Low', u'High')
        self.params[606] = Param(606, 0, u'Sleeve Length', u'jacket', .8, 0, 1, .8, '', u'Short', u'Long')
        self.params[607] = Param(607, 0, u'Collar Front', u'jacket', .8, 0, 1, .8, '', u'Low', u'High')
        self.params[608] = Param(608, 0, u'bottom length lower', u'jacket', .8, 0, 1, .8, u'Jacket Length', u'Short', u'Long')
        self.params[609] = Param(609, 0, u'open jacket', u'jacket', .2, 0, 1, .2, u'Open Front', u'Open', u'Closed')
        self.params[614] = Param(614, 1, u'Waist Height Cloth', u'pants', .8, 0, 1, .8, '', '', '')
        self.params[615] = Param(615, 1, u'Pants Length Cloth', u'pants', .8, 0, 1, .8, '', '', '')
        self.params[616] = Param(616, 0, u'Shoe Height', u'shoes', 0.1, 0, 1, 0.1, '', u'Short', u'Tall')
        self.params[617] = Param(617, 0, u'Socks Length', u'socks', 0.35, 0, 1, 0.35, '', u'Short', u'Long')
        self.params[619] = Param(619, 0, u'Pants Length', u'underpants', .3, 0, 1, .3, '', u'Short', u'Long')
        self.params[620] = Param(620, 1, u'bottom length upper', u'jacket', .8, 0, 1, .8, '', u'hi cut', u'low cut')
        self.params[621] = Param(621, 1, u'bottom length lower', u'jacket', .8, 0, 1, .8, '', u'hi cut', u'low cut')
        self.params[622] = Param(622, 1, u'open upper', u'jacket', .8, 0, 1, .8, '', u'closed', u'open')
        self.params[623] = Param(623, 1, u'open lower', u'jacket', .8, 0, 1, .8, '', u'open', u'closed')
        self.params[624] = Param(624, 0, u'Pants Waist', u'underpants', .8, 0, 1, .8, '', u'Low', u'High')
        self.params[625] = Param(625, 0, u'Leg_Pantflair', u'pants', 0.0, 0, 1.5, 0.0, u'Cuff Flare', u'Tight Cuffs', u'Flared Cuffs')
        self.params[626] = Param(626, 1, u'Big_Chest', u'shape', 0.0, 0, 1, 0.0, u'Chest Size', u'Small', u'Large')
        self.params[627] = Param(627, 1, u'Small_Chest', u'shape', 0.0, 0, 1, 0.0, u'Chest Size', u'Large', u'Small')
        self.params[628] = Param(628, 1, u'Displace_Loose_Upperbody', u'shirt', 0, 0, 1, 0, u'Shirt Fit', '', '')
        self.params[629] = Param(629, 0, u'Forehead Angle', u'shape', .5, 0, 1, .5, '', u'More Vertical', u'More Sloped')
        self.params[630] = Param(630, 1, u'Forehead_Round', u'shape', 0.0, 0, 1, 0.0, u'Round Forehead', u'Less', u'More')
        self.params[631] = Param(631, 1, u'Forehead_Slant', u'shape', 0.0, 0, 1, 0.0, u'Slanted Forehead', u'Less', u'More')
        self.params[632] = Param(632, 1, u'Express_Open_Mouth', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[633] = Param(633, 1, u'Fat_Head', u'shape', 0.0, 0, 1, 0.0, u'Fat Head', u'Skinny', u'Fat')
        self.params[634] = Param(634, 1, u'Fat_Torso', u'shape', 0.0, 0, 1, 0.0, u'Fat Torso', u'skinny', u'fat')
        self.params[635] = Param(635, 1, u'Fat_Legs', u'shape', 0.0, 0, 1, 0.0, u'Fat Torso', u'skinny', u'fat')
        self.params[637] = Param(637, 0, u'Body Fat', u'shape', 0, 0, 1, 0, '', u'Less Body Fat', u'More Body Fat')
        self.params[638] = Param(638, 0, u'Low_Crotch', u'pants', 0.0, 0, 1.3, 0.0, u'Pants Crotch', u'High and Tight', u'Low and Loose')
        self.params[640] = Param(640, 1, u'Hair_Egg_Head', u'hair', 0.0, -1.3, 1, 0.0, '', '', '')
        self.params[641] = Param(641, 1, u'Hair_Squash_Stretch_Head', u'hair', 0.0, -.5, 1, 0.0, '', '', '')
        self.params[642] = Param(642, 1, u'Hair_Square_Head', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[643] = Param(643, 1, u'Hair_Round_Head', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[644] = Param(644, 1, u'Hair_Forehead_Round', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[645] = Param(645, 1, u'Hair_Forehead_Slant', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[646] = Param(646, 0, u'Egg_Head', u'shape', 0, -1.3, 1, 0, u'Egg Head', u'Chin Heavy', u'Forehead Heavy')
        self.params[647] = Param(647, 0, u'Squash_Stretch_Head', u'shape', 0, -0.5, 1, 0, u'Head Stretch', u'Squash Head', u'Stretch Head')
        self.params[648] = Param(648, 1, u'Scrawny_Torso', u'shape', 0.0, 0, 1.3, 0.0, u'Torso Muscles', u'Regular', u'Scrawny')
        self.params[649] = Param(649, 0, u'Torso Muscles', u'shape', .5, 0, 1, .5, u'Torso Muscles', u'Less Muscular', u'More Muscular')
        self.params[650] = Param(650, 0, u'Eyelid_Corner_Up', u'shape', 0.0, -1.3, 1.2, 0.0, u'Outer Eye Corner', u'Corner Down', u'Corner Up')
        self.params[651] = Param(651, 1, u'Scrawny_Legs', u'shape', 0.0, 0, 1.5, 0.0, u'Scrawny Leg', u'Regular Muscles', u'Less Muscles')
        self.params[652] = Param(652, 0, u'Leg Muscles', u'shape', .5, 0, 1, .5, '', u'Less Muscular', u'More Muscular')
        self.params[653] = Param(653, 0, u'Tall_Lips', u'shape', 0.0, -1, 2, 0.0, u'Lip Fullness', u'Less Full', u'More Full')
        self.params[654] = Param(654, 0, u'Shoe_Toe_Thick', u'shoes', 0.0, 0, 2, 0.0, u'Toe Thickness', u'Flat Toe', u'Thick Toe')
        self.params[655] = Param(655, 1, u'Head Size', u'shape', 0.0, -.25, .10, 0.0, u'Head Size', u'Small Head', u'Big Head')
        self.params[656] = Param(656, 0, u'Crooked_Nose', u'shape', 0.0, -2, 2, 0.0, u'Crooked Nose', u'Nose Left', u'Nose Right')
        self.params[657] = Param(657, 1, u'Smile_Mouth', u'shape', 0.0, 0, 1.4, 0.0, u'Mouth Corner', u'Corner Normal', u'Corner Up')
        self.params[658] = Param(658, 1, u'Frown_Mouth', u'shape', 0.0, 0, 1.2, 0.0, u'Mouth Corner', u'Corner Normal', u'Corner Down')
        self.params[659] = Param(659, 0, u'Mouth Corner', u'shape', .5, 0, 1, .5, '', u'Corner Down', u'Corner Up')
        self.params[660] = Param(660, 1, u'Shear_Head', u'shape', 0, -2, 2, 0, u'Shear Face', u'Shear Left', u'Shear Right')
        self.params[661] = Param(661, 1, u'EyeBone_Head_Shear', u'shape', 0.0, -2, 2, 0.0, '', u'Eyes Shear Left Up', u'Eyes Shear Right Up')
        self.params[662] = Param(662, 0, u'Face Shear', u'shape', .5, 0, 1, .5, '', u'Shear Right Up', u'Shear Left Up')
        self.params[663] = Param(663, 0, u'Shift_Mouth', u'shape', 0, -2, 2, 0, u'Shift Mouth', u'Shift Left', u'Shift Right')
        self.params[664] = Param(664, 0, u'Pop_Eye', u'shape', 0, -2, 2, 0, u'Eye Pop', u'Pop Right Eye', u'Pop Left Eye')
        self.params[665] = Param(665, 0, u'Jaw_Jut', u'shape', 0, -2, 2, 0, u'Jaw Jut', u'Overbite', u'Underbite')
        self.params[666] = Param(666, 1, u'Hands_Relaxed_L', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[667] = Param(667, 1, u'Hands_Point_L', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[668] = Param(668, 1, u'Hands_Fist_L', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[669] = Param(669, 1, u'Hands_Relaxed_R', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[670] = Param(670, 1, u'Hands_Point_R', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[671] = Param(671, 1, u'Hands_Fist_R', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[672] = Param(672, 1, u'Hands_Typing', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[674] = Param(674, 0, u'Hair_Shear_Back', u'hair', -0.3, -1, 2, -0.3, u'Shear Back', u'Full Back', u'Sheared Back')
        self.params[675] = Param(675, 0, u'Hand Size', u'shape', 0.0, -.3, .3, 0.0, '', u'Small Hands', u'Large Hands')
        self.params[676] = Param(676, 0, u'Love_Handles', u'shape', 0, -1, 2, 0, u'Love Handles', u'Less Love', u'More Love')
        self.params[677] = Param(677, 1, u'Scrawny_Torso_Male', u'shape', 0.0, 0, 1.3, 0.0, u'Torso Scrawny', u'Regular', u'Scrawny')
        self.params[678] = Param(678, 0, u'Torso Muscles', u'shape', .5, 0, 1, .5, '', u'Less Muscular', u'More Muscular')
        self.params[679] = Param(679, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .10, 0.0, u'Eyeball Size', u'small eye', u'big eye')
        self.params[680] = Param(680, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .10, 0.0, u'Eyeball Size', u'small eye', u'big eye')
        self.params[681] = Param(681, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .10, 0.0, u'Eyeball Size', u'small eye', u'big eye')
        self.params[682] = Param(682, 0, u'Head Size', u'shape', .5, 0, 1, .5, u'Head Size', u'Small Head', u'Big Head')
        self.params[683] = Param(683, 0, u'Neck Thickness', u'shape', -.15, -.4, .2, -.15, '', u'Skinny Neck', u'Thick Neck')
        self.params[684] = Param(684, 0, u'Breast_Female_Cleavage', u'shape', 0, -.3, 1.3, 0, u'Breast Cleavage', u'Separate', u'Join')
        self.params[685] = Param(685, 0, u'Chest_Male_No_Pecs', u'shape', 0, -.5, 1.1, 0, u'Pectorals', u'Big Pectorals', u'Sunken Chest')
        self.params[686] = Param(686, 1, u'Head_Eyes_Big', u'shape', 0, -2, 2, 0, u'Eye Size', u'Beady Eyes', u'Anime Eyes')
        self.params[687] = Param(687, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .25, 0.0, u'Big Eyeball', u'small eye', u'big eye')
        self.params[688] = Param(688, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .25, 0.0, u'Big Eyeball', u'small eye', u'big eye')
        self.params[689] = Param(689, 1, u'EyeBone_Big_Eyes', u'shape', 0.0, -1, 1, 0.0, '', u'Eyes Back', u'Eyes Forward')
        self.params[690] = Param(690, 0, u'Eye Size', u'shape', .5, 0, 1, .5, u'Eye Size', u'Beady Eyes', u'Anime Eyes')
        self.params[691] = Param(691, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .25, 0.0, u'Big Eyeball', u'small eye', u'big eye')
        self.params[692] = Param(692, 0, u'Leg Length', u'shape', 0.0, -1, 1, 0.0, '', u'Short Legs', u'Long Legs')
        self.params[693] = Param(693, 0, u'Arm Length', u'shape', .6, -1, 1, .6, '', u'Short Arms', u'Long arms')
        self.params[694] = Param(694, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .10, 0.0, u'Eyeball Size', u'small eye', u'big eye')
        self.params[695] = Param(695, 1, u'Eyeball_Size', u'shape', 0.0, -.25, .25, 0.0, u'Big Eyeball', u'small eye', u'big eye')
        self.params[700] = Param(700, 0, u'Lipstick Color', u'skin', .25, 0, 1, .25, '', u'Pink', u'Black')
        self.params[701] = Param(701, 0, u'Lipstick', u'skin', 0.0, 0, .9, 0.0, '', u'No Lipstick', u'More Lipstick')
        self.params[702] = Param(702, 0, u'Lipgloss', u'skin', 0.0, 0, 1, 0.0, '', u'No Lipgloss', u'Glossy')
        self.params[703] = Param(703, 0, u'Eyeliner', u'skin', 0.0, 0, 1, 0.0, '', u'No Eyeliner', u'Full Eyeliner')
        self.params[704] = Param(704, 0, u'Blush', u'skin', 0, 0, .9, 0, '', u'No Blush', u'More Blush')
        self.params[705] = Param(705, 0, u'Blush Color', u'skin', .5, 0, 1, .5, '', u'Pink', u'Orange')
        self.params[706] = Param(706, 0, u'Out Shdw Opacity', u'skin', .6, .2, 1, .6, '', u'Clear', u'Opaque')
        self.params[707] = Param(707, 0, u'Outer Shadow', u'skin', 0.0, 0, .7, 0.0, '', u'No Eyeshadow', u'More Eyeshadow')
        self.params[708] = Param(708, 0, u'Out Shdw Color', u'skin', 0.0, 0, 1, 0.0, '', u'Light', u'Dark')
        self.params[709] = Param(709, 0, u'Inner Shadow', u'skin', 0, 0, 1, 0, '', u'No Eyeshadow', u'More Eyeshadow')
        self.params[710] = Param(710, 0, u'Nail Polish', u'skin', 0.0, 0, 1, 0.0, '', u'No Polish', u'Painted Nails')
        self.params[711] = Param(711, 0, u'Blush Opacity', u'skin', .5, 0, 1, .5, '', u'Clear', u'Opaque')
        self.params[712] = Param(712, 0, u'In Shdw Color', u'skin', 0.0, 0, 1, 0.0, '', u'Light', u'Dark')
        self.params[713] = Param(713, 0, u'In Shdw Opacity', u'skin', .7, .2, 1, .7, '', u'Clear', u'Opaque')
        self.params[714] = Param(714, 0, u'Eyeliner Color', u'skin', 0.0, 0, 1, 0.0, '', u'Dark Green', u'Black')
        self.params[715] = Param(715, 0, u'Nail Polish Color', u'skin', 0.0, 0, 1, 0.0, '', u'Pink', u'Black')
        self.params[750] = Param(750, 0, u'Eyebrow Density', u'hair', 0.7, 0, 1, 0.7, '', u'Sparse', u'Dense')
        self.params[751] = Param(751, 1, u"5 O'Clock Shadow", u'hair', 0.7, 0, 1, 0.7, '', u'Dense hair', u'Shadow hair')
        self.params[752] = Param(752, 0, u'Hair Thickness', u'hair', .5, 0, 1, .5, '', u"5 O'Clock Shadow", u'Bushy Hair')
        self.params[753] = Param(753, 0, u'Saddlebags', u'shape', 0, -0.5, 3, 0, u'Saddle Bags', u'Less Saddle', u'More Saddle')
        self.params[754] = Param(754, 0, u'Hair_Taper_Back', u'hair', 0, -1, 2, 0, u'Taper Back', u'Wide Back', u'Narrow Back')
        self.params[755] = Param(755, 0, u'Hair_Taper_Front', u'hair', 0.05, -1.5, 1.5, 0.05, u'Taper Front', u'Wide Front', u'Narrow Front')
        self.params[756] = Param(756, 0, u'Neck Length', u'shape', 0, -1, 1, 0, '', u'Short Neck', u'Long Neck')
        self.params[757] = Param(757, 0, u'Lower_Eyebrows', u'hair', -1, -4, 2, -1, u'Eyebrow Height', u'Higher', u'Lower')
        self.params[758] = Param(758, 0, u'Lower_Bridge_Nose', u'shape', 0.0, -1.5, 1.5, 0.0, u'Lower Bridge', u'Low', u'High')
        self.params[759] = Param(759, 0, u'Low_Septum_Nose', u'shape', 0.5, -1, 1.5, 0.5, u'Nostril Division', u'High', u'Low')
        self.params[760] = Param(760, 0, u'Jaw_Angle', u'shape', 0, -1.2, 2, 0, u'Jaw Angle', u'Low Jaw', u'High Jaw')
        self.params[761] = Param(761, 1, u'Hair_Volume_Small', u'hair', 0.0, 0, 1.3, 0.0, u'Hair Volume', u'Less', u'More')
        self.params[762] = Param(762, 0, u'Hair_Shear_Front', u'hair', 0.0, 0, 3, 0.0, u'Shear Front', u'Full Front', u'Sheared Front')
        self.params[763] = Param(763, 0, u'Hair Volume', u'hair', .55, 0, 1, .55, '', u'Less Volume', u'More Volume')
        self.params[764] = Param(764, 0, u'Lip_Cleft_Deep', u'shape', 0.0, -.5, 1.2, 0.0, u'Lip Cleft Depth', u'Shallow', u'Deep')
        self.params[765] = Param(765, 0, u'Puffy_Lower_Lids', u'shape', 0.0, -.3, 2.5, 0.0, u'Puffy Eyelids', u'Flat', u'Puffy')
        self.params[766] = Param(766, 1, u'Hands_Salute_R', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[767] = Param(767, 1, u'Bug_Eyed_Head', u'shape', 0, -2, 2, 0, u'Eye Depth', u'Sunken Eyes', u'Bug Eyes')
        self.params[768] = Param(768, 1, u'EyeBone_Bug', u'shape', 0.0, -2, 2, 0.0, '', u'Eyes Sunken', u'Eyes Bugged')
        self.params[769] = Param(769, 0, u'Eye Depth', u'shape', .5, 0, 1, .5, '', u'Sunken Eyes', u'Bugged Eyes')
        self.params[770] = Param(770, 1, u'Elongate_Head', u'shape', 0, -1, 1, 0, u'Shear Face', u'Flat Head', u'Long Head')
        self.params[771] = Param(771, 1, u'Elongate_Head_Hair', u'hair', 0.0, -1, 1, 0.0, '', '', '')
        self.params[772] = Param(772, 1, u'EyeBone_Head_Elongate', u'shape', 0.0, -1, 1, 0.0, '', u'Eyes Short Head', u'Eyes Long Head')
        self.params[773] = Param(773, 0, u'Head Length', u'shape', .5, 0, 1, .5, '', u'Flat Head', u'Long Head')
        self.params[774] = Param(774, 1, u'Shear_Head_Hair', u'hair', 0.0, -2, 2, 0.0, '', '', '')
        self.params[775] = Param(775, 0, u'Body Freckles', u'skin', 0, 0, 1, 0, '', u'Less Freckles', u'More Freckles')
        self.params[776] = Param(776, 1, u'freckles upper', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[777] = Param(777, 1, u'freckles lower', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[778] = Param(778, 1, u'Collar Back Height Cloth', u'shirt', .8, 0, 1, .8, '', '', '')
        self.params[779] = Param(779, 0, u'Collar Back', u'undershirt', .84, 0, 1, .84, '', u'Low', u'High')
        self.params[780] = Param(780, 0, u'Collar Back', u'jacket', .8, 0, 1, .8, '', u'Low', u'High')
        self.params[781] = Param(781, 0, u'Collar Back', u'shirt', .78, 0, 1, .78, '', u'Low', u'High')
        self.params[782] = Param(782, 1, u'Hair_Pigtails_Short', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[783] = Param(783, 1, u'Hair_Pigtails_Med', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[784] = Param(784, 1, u'Hair_Pigtails_Long', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[785] = Param(785, 0, u'Pigtails', u'hair', 0, 0, 1, 0, '', u'Short Pigtails', u'Long Pigtails')
        self.params[786] = Param(786, 1, u'Hair_Ponytail_Short', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[787] = Param(787, 1, u'Hair_Ponytail_Med', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[788] = Param(788, 1, u'Hair_Ponytail_Long', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[789] = Param(789, 0, u'Ponytail', u'hair', 0, 0, 1, 0, '', u'Short Ponytail', u'Long Ponytail')
        self.params[790] = Param(790, 1, u'Hair_Pigtails_Medlong', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[791] = Param(791, 1, u'Hands_Peace_R', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[792] = Param(792, 1, u'Hands_Spread_R', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[793] = Param(793, 1, u'Leg_Longcuffs', u'pants', 0, 0, 3, 0, u'Longcuffs', '', '')
        self.params[794] = Param(794, 1, u'Small_Butt', u'shape', 0.0, 0, 1, 0.0, u'Butt Size', u'Regular', u'Small')
        self.params[795] = Param(795, 0, u'Butt Size', u'shape', .25, 0, 1, .25, u'Butt Size', u'Flat Butt', u'Big Butt')
        self.params[796] = Param(796, 0, u'Pointy_Ears', u'shape', 0.0, -.4, 3, 0.0, u'Ear Tips', u'Flat', u'Pointy')
        self.params[797] = Param(797, 1, u'Fat_Upper_Lip', u'shape', 0.0, 0, 1.5, 0.0, u'Fat Upper Lip', u'Normal Upper', u'Fat Upper')
        self.params[798] = Param(798, 1, u'Fat_Lower_Lip', u'shape', 0.0, 0, 1.5, 0.0, u'Fat Lower Lip', u'Normal Lower', u'Fat Lower')
        self.params[799] = Param(799, 0, u'Lip Ratio', u'shape', .5, 0, 1, .5, u'Lip Ratio', u'More Upper Lip', u'More Lower Lip')
        self.params[800] = Param(800, 0, u'Sleeve Length', u'shirt', .89, 0, 1, .89, '', u'Short', u'Long')
        self.params[801] = Param(801, 0, u'Shirt Bottom', u'shirt', 1, 0, 1, 1, '', u'Short', u'Long')
        self.params[802] = Param(802, 0, u'Collar Front', u'shirt', .78, 0, 1, .78, '', u'Low', u'High')
        self.params[803] = Param(803, 0, u'shirt_red', u'shirt', 1, 0, 1, 1, '', '', '')
        self.params[804] = Param(804, 0, u'shirt_green', u'shirt', 1, 0, 1, 1, '', '', '')
        self.params[805] = Param(805, 0, u'shirt_blue', u'shirt', 1, 0, 1, 1, '', '', '')
        self.params[806] = Param(806, 0, u'pants_red', u'pants', 1, 0, 1, 1, '', '', '')
        self.params[807] = Param(807, 0, u'pants_green', u'pants', 1, 0, 1, 1, '', '', '')
        self.params[808] = Param(808, 0, u'pants_blue', u'pants', 1, 0, 1, 1, '', '', '')
        self.params[809] = Param(809, 1, u'lower_jacket_red', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[810] = Param(810, 1, u'lower_jacket_green', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[811] = Param(811, 1, u'lower_jacket_blue', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[812] = Param(812, 0, u'shoes_red', u'shoes', 1, 0, 1, 1, '', '', '')
        self.params[813] = Param(813, 0, u'shoes_green', u'shoes', 1, 0, 1, 1, '', '', '')
        self.params[814] = Param(814, 0, u'Waist Height', u'pants', 1, 0, 1, 1, '', u'Low', u'High')
        self.params[815] = Param(815, 0, u'Pants Length', u'pants', .8, 0, 1, .8, '', u'Short', u'Long')
        self.params[816] = Param(816, 0, u'Loose Lower Clothing', u'pants', 0.0, 0, 1, 0.0, u'Pants Fit', u'Tight Pants', u'Loose Pants')
        self.params[817] = Param(817, 0, u'shoes_blue', u'shoes', 1, 0, 1, 1, '', '', '')
        self.params[818] = Param(818, 0, u'socks_red', u'socks', 1, 0, 1, 1, '', '', '')
        self.params[819] = Param(819, 0, u'socks_green', u'socks', 1, 0, 1, 1, '', '', '')
        self.params[820] = Param(820, 0, u'socks_blue', u'socks', 1, 0, 1, 1, '', '', '')
        self.params[821] = Param(821, 0, u'undershirt_red', u'undershirt', 1, 0, 1, 1, '', '', '')
        self.params[822] = Param(822, 0, u'undershirt_green', u'undershirt', 1, 0, 1, 1, '', '', '')
        self.params[823] = Param(823, 0, u'undershirt_blue', u'undershirt', 1, 0, 1, 1, '', '', '')
        self.params[824] = Param(824, 0, u'underpants_red', u'underpants', 1, 0, 1, 1, '', '', '')
        self.params[825] = Param(825, 0, u'underpants_green', u'underpants', 1, 0, 1, 1, '', '', '')
        self.params[826] = Param(826, 0, u'underpants_blue', u'underpants', 1, 0, 1, 1, '', '', '')
        self.params[827] = Param(827, 0, u'gloves_red', u'gloves', 1, 0, 1, 1, '', '', '')
        self.params[828] = Param(828, 0, u'Loose Upper Clothing', u'shirt', 0.0, 0, 1, 0.0, u'Shirt Fit', u'Tight Shirt', u'Loose Shirt')
        self.params[829] = Param(829, 0, u'gloves_green', u'gloves', 1, 0, 1, 1, '', '', '')
        self.params[830] = Param(830, 0, u'gloves_blue', u'gloves', 1, 0, 1, 1, '', '', '')
        self.params[831] = Param(831, 1, u'upper_jacket_red', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[832] = Param(832, 1, u'upper_jacket_green', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[833] = Param(833, 1, u'upper_jacket_blue', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[834] = Param(834, 0, u'jacket_red', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[835] = Param(835, 0, u'jacket_green', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[836] = Param(836, 0, u'jacket_blue', u'jacket', 1, 0, 1, 1, '', '', '')
        self.params[840] = Param(840, 0, u'Shirtsleeve_flair', u'shirt', 0.0, 0, 1.5, 0.0, u'Sleeve Looseness', u'Tight Sleeves', u'Loose Sleeves')
        self.params[841] = Param(841, 0, u'Bowed_Legs', u'shape', 0, -1, 1, 0, u'Knee Angle', u'Knock Kneed', u'Bow Legged')
        self.params[842] = Param(842, 0, u'Hip Length', u'shape', 0.0, -1, 1, 0.0, '', u'Short hips', u'Long Hips')
        self.params[843] = Param(843, 1, u'No_Chest', u'shape', 0.0, 0, 1, 0.0, u'Chest Size', u'Some', u'None')
        self.params[844] = Param(844, 0, u'Glove Fingers', u'gloves', 1, .01, 1, 1, '', u'Fingerless', u'Fingers')
        self.params[845] = Param(845, 1, u'skirt_poofy', u'skirt', 0.0, 0, 1.5, 0.0, u'poofy skirt', u'less poofy', u'more poofy')
        self.params[846] = Param(846, 1, u'skirt_loose', u'skirt', 0.0, 0, 1, 0.0, u'loose skirt', u'form fitting', u'loose')
        self.params[847] = Param(847, 1, u'skirt_bowlegs', u'skirt', 0, -1, 1, 0, u'legs skirt', '', '')
        self.params[848] = Param(848, 0, u'skirt_bustle', u'skirt', .2, 0, 2, .2, u'bustle skirt', u'no bustle', u'more bustle')
        self.params[849] = Param(849, 1, u'skirt_belly', '', 0.0, 0, 1, 0.0, u'big belly skirt', '', '')
        self.params[850] = Param(850, 1, u'skirt_saddlebags', '', 0.0, -.5, 3, 0.0, '', '', '')
        self.params[851] = Param(851, 1, u'skirt_chubby', '', 0, 0, 1, 0, '', u'less', u'more')
        self.params[852] = Param(852, 1, u'skirt_bigbutt', '', 0.0, 0, 1, 0.0, u'bigbutt skirt', u'less', u'more')
        self.params[853] = Param(853, 1, u'Bowed_Legs', u'shape', 0.0, -1, 1, 0.0, u'Knee Angle', '', '')
        self.params[854] = Param(854, 1, u'Saddlebags', '', 0.0, -.5, 3, 0.0, '', '', '')
        self.params[855] = Param(855, 1, u'Love_Handles', '', 0, -1, 2, 0, '', '', '')
        self.params[856] = Param(856, 1, u'skirt_lovehandles', '', 0, -1, 2, 0, '', u'less', u'more')
        self.params[857] = Param(857, 1, u'skirt_male', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[858] = Param(858, 0, u'Skirt Length', u'skirt', .4, .01, 1, .4, '', u'Short', u'Long')
        self.params[859] = Param(859, 0, u'Slit Front', u'skirt', 1, 0, 1, 1, '', u'Open Front', u'Closed Front')
        self.params[860] = Param(860, 0, u'Slit Back', u'skirt', 1, 0, 1, 1, '', u'Open Back', u'Closed Back')
        self.params[861] = Param(861, 0, u'Slit Left', u'skirt', 1, 0, 1, 1, '', u'Open Left', u'Closed Left')
        self.params[862] = Param(862, 0, u'Slit Right', u'skirt', 1, 0, 1, 1, '', u'Open Right', u'Closed Right')
        self.params[863] = Param(863, 0, u'skirt_looseness', u'skirt', .333, 0, 1, .333, u'Skirt Fit', u'Tight Skirt', u'Poofy Skirt')
        self.params[866] = Param(866, 1, u'skirt_tight', u'skirt', 0.0, 0, 1, 0.0, u'tight skirt', u'form fitting', u'loose')
        self.params[867] = Param(867, 1, u'skirt_smallbutt', u'skirt', 0.0, 0, 1, 0.0, u'tight skirt', u'form fitting', u'loose')
        self.params[868] = Param(868, 0, u'Shirt Wrinkles', u'shirt', 0, 0, 1, 0, '', '', '')
        self.params[869] = Param(869, 0, u'Pants Wrinkles', u'pants', 0, 0, 1, 0, '', '', '')
        self.params[870] = Param(870, 1, u'Pointy_Eyebrows', u'hair', 0.0, -.5, 1, 0.0, u'Eyebrow Points', u'Smooth', u'Pointy')
        self.params[871] = Param(871, 1, u'Lower_Eyebrows', u'hair', 0.0, -2, 2, 0.0, u'Eyebrow Height', u'Higher', u'Lower')
        self.params[872] = Param(872, 1, u'Arced_Eyebrows', u'hair', 0.0, 0, 1, 0.0, u'Eyebrow Arc', u'Flat', u'Arced')
        self.params[873] = Param(873, 1, u'Bump base', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[874] = Param(874, 1, u'Bump upperdef', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[875] = Param(875, 1, u'jacket upper Wrinkles', u'jacket', 0, 0, 1, 0, '', '', '')
        self.params[876] = Param(876, 1, u'jacket upper Wrinkles', u'jacket', 0, 0, 1, 0, '', '', '')
        self.params[877] = Param(877, 0, u'Jacket Wrinkles', u'jacket', 0, 0, 1, 0, u'Jacket Wrinkles', u'No Wrinkles', u'Wrinkles')
        self.params[878] = Param(878, 1, u'Bump upperdef', u'skin', 0.0, 0, 1, 0.0, '', '', '')
        self.params[879] = Param(879, 0, u'Male_Package', u'shape', 0, -.5, 2, 0, u'Package', u'Coin Purse', u'Duffle Bag')
        self.params[880] = Param(880, 0, u'Eyelid_Inner_Corner_Up', u'shape', 0.0, -1.3, 1.2, 0.0, u'Inner Eye Corner', u'Corner Down', u'Corner Up')
        self.params[899] = Param(899, 1, u'Upper Clothes Shading', u'shirt', 0, 0, 1, 0, '', '', '')
        self.params[900] = Param(900, 1, u'Sleeve Length Shadow', u'shirt', 0.02, 0.02, .87, 0.02, '', '', '')
        self.params[901] = Param(901, 1, u'Shirt Shadow Bottom', u'shirt', 0.0, 0.02, 1, 0.0, '', '', '')
        self.params[902] = Param(902, 1, u'Collar Front Shadow Height', u'shirt', 0.0, 0.02, 1, 0.0, '', '', '')
        self.params[903] = Param(903, 1, u'Collar Back Shadow Height', u'shirt', 0.0, 0.02, 1, 0.0, '', '', '')
        self.params[913] = Param(913, 1, u'Lower Clothes Shading', u'pants', 0, 0, 1, 0, '', '', '')
        self.params[914] = Param(914, 1, u'Waist Height Shadow', u'pants', 0.0, 0.02, 1, 0.0, '', '', '')
        self.params[915] = Param(915, 1, u'Pants Length Shadow', u'pants', 0.0, 0.02, 1, 0.0, '', '', '')
        self.params[921] = Param(921, 0, u'skirt_red', u'skirt', 1, 0, 1, 1, '', '', '')
        self.params[922] = Param(922, 0, u'skirt_green', u'skirt', 1, 0, 1, 1, '', '', '')
        self.params[923] = Param(923, 0, u'skirt_blue', u'skirt', 1, 0, 1, 1, '', '', '')
        self.params[1000] = Param(1000, 1, u'Eyebrow Size Bump', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1001] = Param(1001, 1, u'Eyebrow Size', u'hair', 0.5, 0, 1, 0.5, '', '', '')
        self.params[1002] = Param(1002, 1, u'Eyebrow Density Bump', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1003] = Param(1003, 1, u'Eyebrow Density', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1004] = Param(1004, 1, u'Sideburns bump', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1005] = Param(1005, 1, u'Sideburns', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1006] = Param(1006, 1, u'Moustache bump', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1007] = Param(1007, 1, u'Moustache', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1008] = Param(1008, 1, u'Soulpatch bump', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1009] = Param(1009, 1, u'Soulpatch', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1010] = Param(1010, 1, u'Chin Curtains bump', u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1011] = Param(1011, 1, u'Chin Curtains', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1012] = Param(1012, 1, u"5 O'Clock Shadow bump", u'hair', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1013] = Param(1013, 1, u'Sleeve Length Cloth', u'shirt', 0.0, 0, 0.85, 0.0, '', '', '')
        self.params[1014] = Param(1014, 1, u'Shirt Bottom Cloth', u'shirt', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1015] = Param(1015, 1, u'Collar Front Height Cloth', u'shirt', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1016] = Param(1016, 1, u'Collar Back Height Cloth', u'shirt', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1017] = Param(1017, 1, u'Waist Height Cloth', u'pants', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1018] = Param(1018, 1, u'Pants Length Cloth', u'pants', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1019] = Param(1019, 1, u'Jacket Sleeve Length bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1020] = Param(1020, 1, u'jacket Sleeve Length', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1021] = Param(1021, 1, u'Jacket Collar Front bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1022] = Param(1022, 1, u'jacket Collar Front', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1023] = Param(1023, 1, u'Jacket Collar Back bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1024] = Param(1024, 1, u'jacket Collar Back', '', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1025] = Param(1025, 1, u'jacket bottom length upper bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1026] = Param(1026, 1, u'jacket open upper bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1027] = Param(1027, 1, u'jacket bottom length lower bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1028] = Param(1028, 1, u'jacket open lower bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1029] = Param(1029, 1, u'Sleeve Length Cloth', u'shirt', 0.0, 0, 0.85, 0.0, '', '', '')
        self.params[1030] = Param(1030, 1, u'Shirt Bottom Cloth', u'shirt', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1031] = Param(1031, 1, u'Collar Front Height Cloth', u'shirt', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1032] = Param(1032, 1, u'Collar Back Height Cloth', u'shirt', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1033] = Param(1033, 1, u'jacket bottom length lower bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1034] = Param(1034, 1, u'jacket open lower bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1035] = Param(1035, 1, u'Waist Height Cloth', u'pants', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1036] = Param(1036, 1, u'Pants Length Cloth', u'pants', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1037] = Param(1037, 1, u'jacket bottom length upper bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1038] = Param(1038, 1, u'jacket open upper bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1039] = Param(1039, 1, u'Jacket Sleeve Length bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1040] = Param(1040, 1, u'Jacket Collar Front bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1041] = Param(1041, 1, u'Jacket Collar Back bump', u'jacket', 0.0, 0, 1, 0.0, '', '', '')
        self.params[1042] = Param(1042, 1, u'Sleeve Length', u'undershirt', .4, .01, 1, .4, '', '', '')
        self.params[1043] = Param(1043, 1, u'Sleeve Length bump', u'undershirt', .4, .01, 1, .4, '', '', '')
        self.params[1044] = Param(1044, 1, u'Bottom', u'undershirt', .8, 0, 1, .8, '', '', '')
        self.params[1045] = Param(1045, 1, u'Bottom bump', u'undershirt', .8, 0, 1, .8, '', '', '')
        self.params[1046] = Param(1046, 1, u'Collar Front', u'undershirt', .8, 0, 1, .8, '', '', '')
        self.params[1047] = Param(1047, 1, u'Collar Front bump', u'undershirt', .8, 0, 1, .8, '', '', '')
        self.params[1048] = Param(1048, 1, u'Collar Back', u'undershirt', .8, 0, 1, .8, '', u'Low', u'High')
        self.params[1049] = Param(1049, 1, u'Collar Back bump', u'undershirt', .8, 0, 1, .8, '', '', '')
        self.params[1050] = Param(1050, 1, u'Socks Length bump', u'socks', 0.35, 0, 1, 0.35, '', '', '')
        self.params[1051] = Param(1051, 1, u'Socks Length bump', u'socks', 0.35, 0, 1, 0.35, '', '', '')
        self.params[1052] = Param(1052, 1, u'Shoe Height', u'shoes', 0.1, 0, 1, 0.1, '', '', '')
        self.params[1053] = Param(1053, 1, u'Shoe Height bump', u'shoes', 0.1, 0, 1, 0.1, '', '', '')
        self.params[1054] = Param(1054, 1, u'Pants Length', u'underpants', .3, 0, 1, .3, '', '', '')
        self.params[1055] = Param(1055, 1, u'Pants Length', u'underpants', .3, 0, 1, .3, '', '', '')
        self.params[1056] = Param(1056, 1, u'Pants Waist', u'underpants', .8, 0, 1, .8, '', '', '')
        self.params[1057] = Param(1057, 1, u'Pants Waist', u'underpants', .8, 0, 1, .8, '', '', '')
        self.params[1058] = Param(1058, 1, u'Glove Length', u'gloves', .8, .01, 1, .8, '', '', '')
        self.params[1059] = Param(1059, 1, u'Glove Length bump', u'gloves', .8, .01, 1, .8, '', '', '')
        self.params[1060] = Param(1060, 1, u'Glove Fingers', u'gloves', 1, .01, 1, 1, '', '', '')
        self.params[1061] = Param(1061, 1, u'Glove Fingers bump', u'gloves', 1, .01, 1, 1, '', '', '')



